@@ -930,7 +930,7 @@ void RedundantAlns::add(const AlnRes& res) {
 	assert(!cells_.empty());
 	TRefOff left = res.refoff(), right;
 	const size_t len = res.readExtentRows();
-        const size_t alignmentStart = res.trimmedLeft(true);
+	const size_t alignmentStart = res.trimmedLeft(true);
 	if(!res.fw()) {
 		const_cast<AlnRes&>(res).invertEdits();
 	}
@@ -981,7 +981,7 @@ bool RedundantAlns::overlap(const AlnRes& res) {
 	assert(!cells_.empty());
 	TRefOff left = res.refoff(), right;
 	const size_t len = res.readExtentRows();
-        const size_t alignmentStart = res.trimmedLeft(true);
+	const size_t alignmentStart = res.trimmedLeft(true);
 	if(!res.fw()) {
 		const_cast<AlnRes&>(res).invertEdits();
 	}
@@ -1050,11 +1050,11 @@ void AlnSetSumm::init(
 {
 	assert(rd1 != NULL || rd2 != NULL);
 	assert((rs1 == NULL) == (rs2 == NULL));
-	AlnScore best[2], secbest[2], bestPaired, secbestPaired;
+	AlnScore best[2], bestPaired;
 	size_t szs[2];
-	best[0].invalidate();    secbest[0].invalidate();
-	best[1].invalidate();    secbest[1].invalidate();
-	bestPaired.invalidate(); secbestPaired.invalidate();
+	best[0].invalidate();
+	best[1].invalidate();
+	bestPaired.invalidate();
 	bool paired = (rs1 != NULL && rs2 != NULL);
 	szs[0] = szs[1] = 0;
 	// Set bestPaired and secbestPaired
@@ -1063,20 +1063,6 @@ void AlnSetSumm::init(
 		assert_eq(rs1->size(), rs2->size());
 		szs[0] = szs[1] = rs1->size();
 		assert_gt(szs[0], 0);
-		for(size_t i = 0; i < rs1->size(); i++) {
-			// Combine mate scores into a concordant alignment score by
-			// summing them
-			AlnScore sc = (*rs1)[i].score() + (*rs2)[i].score();
-			if(sc > bestPaired) {
-				secbestPaired = bestPaired;
-				bestPaired = sc;
-				assert(VALID_AL_SCORE(bestPaired));
-			} else if(sc > secbestPaired) {
-				secbestPaired = sc;
-				assert(VALID_AL_SCORE(bestPaired));
-				assert(VALID_AL_SCORE(secbestPaired));
-			}
-		}
 	}
 	// Set best[] and secbest[]
 	for(int j = 0; j < 2; j++) {
@@ -1085,27 +1071,9 @@ void AlnSetSumm::init(
 			continue;
 		}
 		szs[j] = rs->size();
-		for(size_t i = 0; i < rs->size(); i++) {
-			AlnScore sc = (*rs)[i].score();
-			if(sc > best[j]) {
-				secbest[j] = best[j];
-				best[j] = sc;
-				assert(VALID_AL_SCORE(best[j]));
-			} else if(sc > secbest[j]) {
-				secbest[j] = sc;
-				assert(VALID_AL_SCORE(best[j]));
-				assert(VALID_AL_SCORE(secbest[j]));
-			}
-		}
 	}
 	if(szs[0] > 0 || szs[1] > 0) {
 		init(
-			best[0],
-			secbest[0],
-			best[1],
-			secbest[1],
-			bestPaired,
-			secbestPaired,
 			(szs[0] == 0) ? 0 : (szs[0] - 1),
 			(szs[1] == 0) ? 0 : (szs[1] - 1),
 			paired,
@@ -1130,7 +1098,7 @@ bool AlnFlags::printYF(BTString& o, bool first) const {
 	else if(!nfilt_  ) flag = "NS";
 	else if(!scfilt_ ) flag = "SC";
 	else if(!qcfilt_ ) flag = "QC";
-	if(flag > 0) {
+	if(flag[0] != '\0') {
 		if(!first) o.append('\t');
 		o.append("YF:Z:");
 		o.append(flag);
