@@ -1,16 +1,29 @@
 /*
- * bowtie_build_main.cpp
+ * Copyright 2011, Ben Langmead <langmea@cs.jhu.edu>
  *
- *  Created on: Sep 16, 2009
- *      Author: Ben Langmead
+ * This file is part of Bowtie 2.
+ *
+ * Bowtie 2 is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Bowtie 2 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Bowtie 2.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <iostream>
 #include <fstream>
 #include <string.h>
 #include <stdlib.h>
-#include <vector>
 #include "tokenize.h"
+#include "ds.h"
+#include "mem_ids.h"
 
 using namespace std;
 
@@ -35,7 +48,7 @@ int main(int argc, const char **argv) {
 		char buf[4096];
 		int lastret = -1;
 		while(in.getline(buf, 4095)) {
-			vector<string> args;
+			EList<string> args(MISC_CAT);
 			args.push_back(string(argv[0]));
 			tokenize(buf, " \t", args);
 			const char **myargs = (const char**)malloc(sizeof(char*)*args.size());
@@ -43,7 +56,7 @@ int main(int argc, const char **argv) {
 				myargs[i] = args[i].c_str();
 			}
 			if(args.size() == 1) continue;
-			lastret = bowtie_build(args.size(), myargs);
+			lastret = bowtie_build((int)args.size(), myargs);
 			free(myargs);
 		}
 		if(lastret == -1) {
